@@ -1,3 +1,5 @@
+from partis.schema_meta.base import (
+  Loc )
 
 #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 def as_load( loads ):
@@ -7,6 +9,10 @@ def as_load( loads ):
     *args,
     **kwargs ):
 
+    if 'loc' not in kwargs:
+      kwargs['loc'] = Loc(
+        filename = file )
+
     with open( file, 'rb' ) as fp:
       src = fp.read()
 
