@@ -17,6 +17,7 @@ from partis.pyproj import (
 from pygments.lexers import get_lexer_by_name
 
 from partis.utils import (
+  cached_property,
   adict,
   odict,
   rdict,
@@ -98,7 +99,6 @@ class ProviderSupport:
     self._name = str(name)
     self._lexer = lexer
     self._doc = inspect.cleandoc( doc )
-    self._p_schema_hash_node = None
 
   #-----------------------------------------------------------------------------
   @property
@@ -132,23 +132,19 @@ class ProviderSupport:
     return hash(self._name)
 
   #-----------------------------------------------------------------------------
-  @property
+  @cached_property
   def schema_hash_node( self ):
     """urlsafe Base64 encoded hash
     """
 
-    if not self._p_schema_hash_node:
-
-      self._p_schema_hash_node = hash_sha256( self._name.encode('utf-8') )[0]
-
-    return self._p_schema_hash_node
+    return hash_sha256( self._name.encode('utf-8') )[0]
 
 #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 class Provider( metaclass = ProviderMeta ):
 
   #-----------------------------------------------------------------------------
   def __init__( self ):
-    self._p_schema_hash_node = None
+    pass
 
   #-----------------------------------------------------------------------------
   @property
@@ -190,9 +186,9 @@ class Provider( metaclass = ProviderMeta ):
   def lint( self,
     schema,
     src,
+    loc = None,
     locals = None,
     module = None,
-    lint = None,
     logger = None ):
     """Lints source data
 
@@ -202,6 +198,7 @@ class Provider( metaclass = ProviderMeta ):
       Schema for evaluation result
     src : object
       The source data to be evaluated
+    loc : None | Loc
     locals : dict[ str, object ] | None
       Defines local variables to use for evaluation.
     module : None | str | ModuleType
@@ -220,6 +217,7 @@ class Provider( metaclass = ProviderMeta ):
   def eval( self,
     schema,
     src,
+    loc = None,
     locals = None,
     module = None,
     logger = None ):
@@ -231,6 +229,7 @@ class Provider( metaclass = ProviderMeta ):
       Schema for evaluation result
     src : object
       The source data to be evaluated
+    loc : None | Loc
     locals : dict[ str, object ] | None
       Defines local variables to use for evaluation.
     module : None | str | ModuleType
@@ -249,18 +248,14 @@ class Provider( metaclass = ProviderMeta ):
     raise NotImplementedError("Derived provider must implement `eval`")
 
   #-----------------------------------------------------------------------------
-  @property
+  @cached_property
   def schema_hash_node( self ):
     """urlsafe Base64 encoded hash
     """
 
-    if not self._p_schema_hash_node:
-
-      self._p_schema_hash_node = hash_sha256( '\n'.join([
-        f'{k}, {v.schema_hash_node}'
-        for k,v in self.supported.items()  ]).encode('utf-8') )[0]
-
-    return self._p_schema_hash_node
+    return hash_sha256( '\n'.join([
+      f'{k}, {v.schema_hash_node}'
+      for k,v in self.supported.items()  ]).encode('utf-8') )[0]
 
 
 #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
@@ -317,9 +312,9 @@ class ProviderUnion( Provider ):
   def lint( self,
     schema,
     src,
+    loc = None,
     locals = None,
     module = None,
-    lint = None,
     logger = None ):
 
     supported = self.check( src )
@@ -332,6 +327,7 @@ class ProviderUnion( Provider ):
     return self._p_support_map[support].lint(
       schema = schema,
       src = src,
+      loc = loc,
       locals = locals,
       module = module,
       logger = logger )
@@ -340,6 +336,7 @@ class ProviderUnion( Provider ):
   def eval( self,
     schema,
     src,
+    loc = None,
     locals = None,
     module = None,
     logger = None ):
@@ -354,21 +351,18 @@ class ProviderUnion( Provider ):
     return self._p_support_map[support].eval(
       schema = schema,
       src = src,
+      loc = loc,
       locals = locals,
       module = module,
       logger = logger )
 
   #-----------------------------------------------------------------------------
-  @property
+  @cached_property
   def schema_hash_node( self ):
 
-    if not self._p_schema_hash_node:
-
-      self._p_schema_hash_node = hash_sha256( '\n'.join([
-        f'{k}, {v.schema_hash_node}'
-        for k,v in self.supported.items()  ]).encode('utf-8') )[0]
-
-    return self._p_schema_hash_node
+    return hash_sha256( '\n'.join([
+      f'{k}, {v.schema_hash_node}'
+      for k,v in self.supported.items()  ]).encode('utf-8') )[0]
 
 #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 class EvaluatedContextMeta( ABCMeta ):
@@ -428,8 +422,7 @@ class EvaluatedContextMeta( ABCMeta ):
     namespace = { **namespace, **dict(
       _p_doc = doc,
       __doc__ = StringFunction(),
-      _p_id = id,
-      _p_schema_hash_node = None ) }
+      _p_id = id ) }
 
     cls = super().__new__( mcls, name, bases, namespace )
 
@@ -485,16 +478,12 @@ class EvaluatedContextMeta( ABCMeta ):
     return cls._p_id != str(other)
 
   #-----------------------------------------------------------------------------
-  @property
+  @cached_property
   def schema_hash_node( cls ):
     """urlsafe Base64 encoded hash
     """
 
-    if not cls._p_schema_hash_node:
-
-      cls._p_schema_hash_node = hash_sha256(cls._p_id.encode('utf-8'))[0]
-
-    return cls._p_schema_hash_node
+    return hash_sha256(cls._p_id.encode('utf-8'))[0]
 
 #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 class EvaluatedContext(
@@ -669,8 +658,7 @@ class EvaluatedMeta( ABCMeta ):
     namespace = odict( **namespace,
       _p_num_subclass = 0,
       _p_provider = provider,
-      _p_context = context,
-      _p_schema_hash_node = None )
+      _p_context = context )
 
     cls = super().__new__( mcls, name, bases, namespace )
 
@@ -839,14 +827,10 @@ class EvaluatedMeta( ABCMeta ):
     return lines
 
   #-----------------------------------------------------------------------------
-  @property
+  @cached_property
   def schema_hash_node( cls ):
     """urlsafe Base64 encoded hash
     """
 
-    if not cls._p_schema_hash_node:
-
-      cls._p_schema_hash_node = hash_sha256(
-        '\n'.join( cls._schema_hash_node() ).encode('utf-8') )[0]
-
-    return cls._p_schema_hash_node
+    return hash_sha256(
+      '\n'.join( cls._schema_hash_node() ).encode('utf-8') )[0]
